

#ifndef __SVector__
#define __SVector__

#include "Object.h"
#include "Enumeration.h"
#include "Vector.h"

/**
 * The Vector class implements a growable array of objects. Like an array, it contains components 
 * that can be accessed using an integer index. However, the size of a Vector can grow or shrink as 
 * needed to accommodateadding and removing items after the Vector has been created. 
 *
 * Each vector tries to optimize storage management by maintaining a capacity and a capacityIncrement.
 * The capacity is always at least as large as the vector size; it is usually larger because as 
 * components are added to the vector, the vector's storage increases in chunks the size of 
 * capacityIncrement. An application can increase the capacity of a vector before inserting a large 
 * number of components; this reduces the amount of incremental reallocation. 
 */

class SVector : public Vector
{
private:

public:
  SVector(); /** Generic Constructor */
  SVector(int); /**Constructor with a given amount of Capacity */
  SVector(int, int); /** Constructor with a given amount of Capacity and incrementSize */ 
  SVector(SVector &); /**CopyConstructor */
  virtual ~SVector(); /** Generic Destructor */
  void addElement(Object *); /** Adds the specified component to the end of this SVector */
  void setElementAt(Object *, int); /**Overwrites a given element at Position x **/
  void insertElementAt(Object *, int);/** Inserts the specified object as a component in this SVector  at the specified index. */
  bool removeElementAt(int); /** Deletes the component at the specified index. */ 
  virtual SVector *clone(); /**Returns a clone of the SVector */
  virtual String toString();
};

ostream &operator<<(ostream &,SVector &);

#endif
