#include "TrlGui.h"
#include <stdlib.h>

TrlGui::TrlGui(QWidget *parent, const char *name)
: QWidget(parent, name)
{
  QFont gfnt;
  String path;

  this->resize(250, 140);
  this->setMinimumHeight(140);

  //menubarentries
  file = new QPopupMenu();
  gfnt = QFont("Helvetica", 12, 50);
  file->setFont(gfnt);
  file->insertItem("Open Dictionary [&german->english]", this, SLOT(openDE()));
  file->insertItem("Open Dictionary [&english->german]", this, SLOT(openED()));
  file->insertSeparator();
  file->insertItem("&Quit", this, SLOT(quitSelected()));
  
  //menubarentries
  help = new QPopupMenu();
  gfnt = QFont("Helvetica", 12, 50);
  help->setFont(gfnt);
  help->insertItem("Help", this, SLOT(helpSelected()));
  help->insertItem("About", this, SLOT(aboutSelected()));

  //menubar
  bar = new QMenuBar(this);
  bar->setGeometry(0, 0, 250, 12);
  gfnt = QFont("Helvetica", 12, 50);
  gfnt.setItalic(false);
  gfnt.setUnderline(false);
  gfnt.setStrikeOut(false);
  gfnt.setFixedPitch(false);
  bar->setFont(gfnt);
  bar->setFrameStyle(QFrame::Panel | QFrame::Raised);
  bar->insertItem("&File",file);
  bar->insertSeparator();
  bar->setSeparator(QMenuBar::InWindowsStyle);
  bar->insertItem("&Help", help);

  
  //set language
  language = new QButtonGroup(this);
  language->setGeometry(0, 25, 175, 30);
  language->setMinimumSize(175, 30);
  language->setStyle((GUIStyle)1);
  language->setFrameStyle(51);
  language->setLineWidth(2);
  language->setExclusive(true);
   
  //german -> english
  german = new QRadioButton(language);
  german->setGeometry(10, 8, 80, 15);
  german->setMinimumSize(80, 15);
  gfnt = QFont("Helvetica", 11, 50);
  gfnt.setItalic(false);
  gfnt.setUnderline(false);
  gfnt.setStrikeOut(false);
  gfnt.setFixedPitch(false);
  german->setFont(gfnt);
  german->setText("ger -> eng");
  german->setChecked(false);

  //english -> german
  english = new QRadioButton(language);
  english->setGeometry(90, 8, 80, 15);
  english->setMinimumSize(80, 15);
  gfnt = QFont("Helvetica", 11, 50);
  gfnt.setItalic(false);
  gfnt.setUnderline(false);
  gfnt.setStrikeOut(false);
  gfnt.setFixedPitch(false);
  english->setFont(gfnt);
  english->setText("eng -> ger");
  english->setChecked(true);


  //searchtype
  type = new QComboBox(this);
  type->setGeometry(175, 25, 75, 30);
  type->setMinimumSize(75, 30);
  gfnt = QFont("Helvetica", 11, 50);
  gfnt.setItalic(false);
  gfnt.setUnderline(false);
  gfnt.setStrikeOut(false);
  gfnt.setFixedPitch(false);
  type->setFont(gfnt);
  type->setStyle((GUIStyle)1);
  type->setFocusPolicy((QWidget::FocusPolicy)3);
  type->insertItem("exact");
  type->insertItem("near");
  type->insertItem("near + list");
  type->setCurrentItem(1);


  //word
  query = new QLineEdit(this);
  query->setMinimumSize(125, 30);
  gfnt = QFont("Helvetica", 12, 50);
  gfnt.setItalic(false);
  gfnt.setUnderline(false);
  gfnt.setStrikeOut(false);
  gfnt.setFixedPitch(false);
  query->setFont(gfnt);
  query->setStyle((GUIStyle)1);

  
  //last
  last = new QComboBox(TRUE,this);
  last->setMinimumSize(125, 30);
  last->setStyle((GUIStyle)1);
  last->setFocusPolicy((QWidget::FocusPolicy)3);
  gfnt = QFont("Helvetica", 12, 50);
  gfnt.setItalic(false);
  gfnt.setUnderline(false);
  gfnt.setStrikeOut(false);
  gfnt.setFixedPitch(false);
  last->setFont(gfnt);

  //splitted results
  split = new QSplitter(this);
  split->setOrientation(QSplitter::Vertical);
  split->setOpaqueResize(true);

  //query results
  edit = new QMultiLineEdit(split);
  edit->setMinimumSize(250, 60);
  gfnt = QFont("Helvetica", 12, 50);
  gfnt.setUnderline(false);
  gfnt.setStrikeOut(false);
  gfnt.setFixedPitch(false);
  edit->setFont(gfnt);
  edit->setStyle((GUIStyle)1);
  edit->setReadOnly(true);


  //query near's
  near = new QMultiLineEdit(split);
  near->setStyle((GUIStyle)1);
  near->insertLine("");
  near->setReadOnly(true);
  near->setFont(gfnt);
  near->setStyle((GUIStyle)1);
  near->setMaximumSize(2500, 0);
  split->setResizeMode(near, QSplitter::KeepSize);

  //vertical layout
  layer = new QVBoxLayout(this, 1);
    //horizontal layout
    hlay1 = new QHBoxLayout();
    //horizontal layout
    hlay2 = new QHBoxLayout();
  layer->setMenuBar(bar);
  layer->addLayout(hlay1);
  layer->addLayout(hlay2);
    hlay1->addWidget(language);
    hlay1->addWidget(type, 10);
    hlay2->addWidget(query);
    hlay2->addWidget(last);
  layer->addWidget(split,10);
  layer->activate();

  QObject::connect(query, SIGNAL(returnPressed()), this, SLOT(queryWord()));
  QObject::connect(last, SIGNAL(activated(const char *)), this, SLOT(translate(const char *)));
  QObject::connect(type, SIGNAL(activated(const char *)), this, SLOT(changeSearchType(const char *)));
  QObject::connect(language, SIGNAL(clicked(int)), this, SLOT(changeLanguage(int)));

  path = getenv("TRLPATH");
  if (path.length() < 1)
  {
      path = getenv("PWD");
      path = "Set environment variable TRLPATH to dictionaries directory!\n\nNow I try to use: " + path + "/dictionary/\nFor further information read README"; 
      QMessageBox::about(this, 	"TRLPATH", (const char*) path);
      path = "./dictionary/";
  }
  else
  {
      path = path + "/";
  }
  germanTOenglish = new Diction(path+"de");
  englishTOgerman = new Diction(path+"ed");
  active_dictionary = englishTOgerman;
  lsearchtype = em_near;
}

TrlGui::~TrlGui()
{
   delete(file);
   delete(bar);
   delete(german);
   delete(english);
   delete(language);
   delete(type);
   delete(query);
   delete(last);
   delete(edit);
   delete(near);
   delete(germanTOenglish);
   delete(englishTOgerman);
   delete(hlay1);
   delete(hlay2);
   delete(layer);
}
void TrlGui::queryWord()
{
   translate(query->text());
   last->insertItem(query->text(),0);
   query->setText("");
}
void TrlGui::translate(const char *word)
{
   String lword = word;
   String buf;
   DWord result; 

   switch (lsearchtype)
   {
      case em_exact:
      			result = active_dictionary->getWord(lword);
			buf = result.getPeerWord();
			buf.gsub("/","\n");
			edit->setText((const char*)("["+result.getFirstWord()+"]"));
			edit->append((const char*)buf);
			break;
      case em_near:
                        result = active_dictionary->getNear(lword);
			buf = result.getPeerWord();
			buf.gsub("/","\n");
			edit->setText((const char*)("["+result.getFirstWord()+"]"));
			edit->append((const char*)buf);
			break;
      case em_list:	
                        result = active_dictionary->getNear(lword);
			buf = result.getPeerWord();
			buf.gsub("/","\n");
			edit->setText((const char*)("["+result.getFirstWord()+"]"));
			edit->append((const char*)buf);
			near->setText((const char*)(active_dictionary->getNarrow(lword))->toString());
      			break;
      default:
                        result = active_dictionary->getNear(lword);
			edit->setText((const char*)result.getPeerWord());
   }

}

void TrlGui::changeSearchType(const char *type)
{
   String str_type=type;

   if (str_type == "exact") 
   {
       lsearchtype = em_exact;
       near->setText("");
       near->setMinimumSize(250, 0);
       near->setMaximumSize(2500, 0);
       split->setResizeMode(near, QSplitter::KeepSize);
   }
   if (str_type == "near") 
   {
       lsearchtype = em_near;
       near->setText("");
       near->setMinimumSize(250, 0);
       near->setMaximumSize(2500, 0);
       split->setResizeMode(near, QSplitter::KeepSize);
   }
   if (str_type == "near + list") 
   {
       near->setMinimumSize(250, 60);
       near->setMaximumSize(2500, 2000);
       split->setResizeMode(near, QSplitter::Stretch);
       lsearchtype = em_list;
       translate(last->currentText());
   }
}
void TrlGui::changeLanguage(int id)
{
   switch (id)
   {
      case 0:  active_dictionary=germanTOenglish;
               break;
      case 1:  active_dictionary=englishTOgerman;
               break;
   }
}
void TrlGui::quitSelected()
{
   emit quit_trl();
}
void TrlGui::openDE()
{
   QString filename = QFileDialog::getOpenFileName(0,"*", this);
   if (!filename.isEmpty())
   {
       delete (germanTOenglish);
       germanTOenglish = new Diction((String)filename);
   }
}
void TrlGui::openED()
{
   QString filename = QFileDialog::getOpenFileName(0,"*", this);
   if (!filename.isEmpty())
   {
       delete (englishTOgerman);
       englishTOgerman = new Diction((String)filename);
   }
}
void TrlGui::aboutSelected()
{

    QMessageBox::about(this, 	"About trl Ver 0.3",
	    			"trl Ver 0.3\nwritten by Matthias Veit\neMail: mveit@cs.tu-berlin.de\n");
}
void TrlGui::helpSelected()
{
    edit->setText("Hopefully no Help is needed   :).\n\nOtherwise refer to the README file\ninside your installation directory.\n\nPlease note there is no warranty for\nany consequences by using trl,\nso use at your own risk"); 
    near->setText("");
}
